package com.jnym.mgmt.client.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.jnym.mgmt.client.entity.CustomPlatform;
import com.jnym.mgmt.client.enums.WebHookType;
import com.jnym.mgmt.client.service.CustomPlatformService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/customPlatform")
@Api(value = "自定义配置", tags = "自定义配置接口")
@Slf4j
public class CustomPlatformController {

    @Autowired
    private CustomPlatformService customPlatformService;


    @GetMapping("/selectList")
    @ApiOperation(value = "自定义平台类型下拉列表")
    public R selectList() {
            List<Map<String, String>> data = new ArrayList<>();
			WebHookType[] values = WebHookType.values();
			for (WebHookType value : values) {
                Map<String, String> map = new HashMap<>();
				map.put("val",value.getValue());
				map.put("msg",value.getMessage());
                data.add(map);
			}
        return R.data(data);
    }


    @PostMapping("save")
    @ApiOperation(value = "自定义配置新增")
    public R createPlatform(@RequestBody CustomPlatform customPlatform) {
        customPlatform.setCreateTime(new Date());
        customPlatform.setCreator(AuthUtil.getNickName());
        return R.data(customPlatformService.save(customPlatform));
    }

    @GetMapping("/detail")
    @ApiOperation(value = "自定义配置详情")
    public R getPlatformById(Long id) {
        CustomPlatform customPlatform = customPlatformService.getById(id);
        return R.data(customPlatform);
    }


    @PutMapping("/update")
    @ApiOperation(value = "自定义配置编辑")
    public R updatePlatform(@RequestBody CustomPlatform updatedPlatform) {
        if(updatedPlatform.getId() == null){
            return R.fail("id不嫩为空");
        }
        return R.data(customPlatformService.updateById(updatedPlatform));
    }

    @DeleteMapping("/delete")
    @ApiOperation(value = "自定义配置删除")
    public R deletePlatform(Long id) {
        return R.data(customPlatformService.removeById(id));
    }

    @GetMapping("/page")
    @ApiOperation(value = "自定义配置分页列表")
    public R<IPage<CustomPlatform>> PlatformPage(String platformType,String platformName,Query query) {
        LambdaQueryWrapper<CustomPlatform> queryWrapper = new LambdaQueryWrapper<>();
        if(StringUtils.isNotBlank(platformType)){
            queryWrapper.eq(CustomPlatform::getPlatformType,platformType);
        }
        if(StringUtils.isNotBlank(platformName)){
            queryWrapper.like(CustomPlatform::getPlatformName,platformName);
        }
        IPage<CustomPlatform> res = customPlatformService.page(Condition.getPage(query),queryWrapper);
        return R.data(res);
    }

    @GetMapping("/customPlatformList")
    @ApiOperation(value = "自定义配置下拉列表")
    public R customPlatformList() {
        List<CustomPlatform> list = customPlatformService.list(new LambdaQueryWrapper<CustomPlatform>()
                .isNotNull(CustomPlatform::getId)
        );
        List<Map<String, Object>> data = list.stream()
                .map(obj -> {
                    Map<String, Object> map = new HashMap<>();
                    map.put("id", obj.getId());
                    map.put("name", obj.getPlatformName());
                    return map;
                }).collect(Collectors.toList());
        return R.data(data);
    }

}
