package com.jnym.mgmt.client.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jnym.mgmt.client.vo.*;
import com.jnym.mgmt.client.service.IamService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.modules.system.entity.Tenant;
import org.springblade.modules.system.vo.UserVO;
import org.springblade.system.client.dto.IamGetUserByIdsParams;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/yx/iam")
@Api(tags = "iam相关接口")
public class IamController {

    @Autowired
    private IamService iamService;


    @ApiOperation("当前租户的组织树")
    @GetMapping("/orgTree")
    public R<Object> orgTree() {
        return R.data(iamService.orgTree());
    }

    @ApiOperation("通过部门获取所有用户")
    @GetMapping("getUserByDept/{id}")
    public R<Object> getUserByDept(@PathVariable("id") long departmentId) {
        return R.data(iamService.getUserByDept(departmentId));
    }


    @ApiOperation("通过名字获取租户信息")
    @GetMapping("/getTenantList")
    public R<IPage<TenantModel>> getTenantList(GetTenantListParams params){
        return R.data(iamService.getTenantList(params));
    }

    @ApiOperation("用户查询分页")
    @GetMapping("/userPage")
    public R<Page<UserVO>> userPage(GetIamUserPageParams params){
        return R.data(iamService.userPage(params));
    }

    @ApiOperation("个人租户 企业租户")
    @GetMapping("/tenantPage")
    public R<IPage<Tenant>> tenantPage(TenantPageParams params){
        return R.data(iamService.tenantPage(params));
    }

    @ApiOperation("根据账号或者手机号精确查询用户")
    @GetMapping("/getUserByUsernameOrPhone")
    public R<List<UserVO>> getUserByUsernameOrPhone(String keyName,Boolean ignoreTenant){
        return R.data(iamService.getUserByUsernameOrPhone(keyName,ignoreTenant));
    }


    @ApiOperation("根据账号、手机或者邮箱精确查询用户(不包含子用户)")
    @GetMapping("/exactUserCommon")
    public R<UserVO> exactUserCommon(String keyName){
        return R.data(iamService.exactUserCommon(keyName));
    }

    @ApiOperation("管理端-根据账号或者名称精确查询用户信息")
    @GetMapping("/manageGetByKeyName")
    public R<List<UserVO>> manageGetByKeyName(String keyName){
        return R.data(iamService.manageGetByKeyName(keyName));
    }

    @ApiOperation("获取角色列表")
    @GetMapping("/getRoleList")
    public R<GetIamRoleListModel> getRoleList(GetIamRoleListParams params){
        return R.data(iamService.getRoleList(params));
    }

    /**
     * 通过租户id集合查询租户信息
     */
    @ApiOperation("通过租户id集合查询租户信息")
    @PostMapping( "/tenantNameList")
    public R<List<TenantModel>> getTenantByTenantIds(@RequestBody List<String> tenantIds){
        return R.data(iamService.getTenantNames(tenantIds));
    }

    @GetMapping("/getUserByAccountAndNickName")
    @ApiOperation("根据昵称或者用户名查询用户信息")
    public R<Page<UserVO>> getUserByAccountAndNickName(GetUserByAccountAndNickNameParams params){
        return R.data(iamService.getUserByAccountAndNickName(params));
    }

    @ApiOperation("批量获取用户信息")
    @PostMapping("/getUserInfos")
    public R<List<UserModel>> getByIds(@RequestBody List<Long> userIds) {
        IamGetUserByIdsParams iamGetUserByIdsParams = new IamGetUserByIdsParams();
        iamGetUserByIdsParams.setHide(0);
        iamGetUserByIdsParams.setUserIds(userIds);
        iamGetUserByIdsParams.setInstId(AuthUtil.getInstId());
        return R.data(iamService.getByUserIds(iamGetUserByIdsParams));
    }

}
