package com.jnym.mgmt.client.controller;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.modules.auth.utils.TokenUtil;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.entity.UserInfo;
import org.springframework.web.bind.annotation.*;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

@RestController
@RequestMapping("/9n1m")
@Api(value = "Iam用户信息", tags = "Iam用户信息接口")
@Slf4j
public class IamUserController {


    @PostMapping("/getInstToken")
    public Kv getInstToken(@RequestBody JSONObject authToken) throws Exception {
        Kv authInfo = Kv.create();
        String token = authToken.getString("authToken");
        if(StringUtils.isBlank(token)){
            return authInfo.set("error_code", 400).set("error_msg", "authToken不能为空");
        }
        String url = "https://91iam-inst.9n1m.com/api/users-service/users/currentUser";
        URL obj = new URL(url);
        HttpURLConnection con = (HttpURLConnection) obj.openConnection();

        // 添加请求头
        con.setRequestMethod("GET");
        con.setRequestProperty("Authorization", token);

        // 发送请求
        int responseCode = con.getResponseCode();
        log.info("Response Code : {}",responseCode);
        // 读取响应
        BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
        String inputLine;
        StringBuilder response = new StringBuilder();
        while ((inputLine = in.readLine()) != null) {
            response.append(inputLine);
        }
        in.close();
        JSONObject jsonObject = JSONObject.parseObject(response.toString());
        JSONObject data = jsonObject.getJSONObject("data");
        return sysToken(data);
    }

    public Kv sysToken(JSONObject data) {
        User user = new User();
        UserInfo userInfo = new UserInfo();
        user.setAccount(data.getString("username"));
        user.setTenantId(data.getString("instId"));
        user.setId(data.getLong("userId"));
        userInfo.setUser(user);
        return TokenUtil.createAuthInfo(userInfo);
    }


    /**
     * 文档地址 ： 	获取部门
     * 	https://91iam-inst.9n1m.com/api/doc.html#/%E8%BA%AB%E4%BB%BD%E7%AE%A1%E7%90%86%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%BB%84%E7%BB%87%E7%AE%A1%E7%90%86/list_1
     * @param id
     * @param orgName
     * @param instId
     * @param pageSize
     * @param pageNum
     * @param orderByColumn
     * @param isAsc
     * @return
     * @throws Exception
     */
    @GetMapping("/getOrgList")
    @ApiOperation(value = "获取iam租户所属部门（组织）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = false),
            @ApiImplicitParam(name = "orgName", value = "简称",  required = false),
            @ApiImplicitParam(name = "instId", value = "租户ID",  required = false),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = false),
            @ApiImplicitParam(name = "pageNum", value = "当前页数", required = false),
            @ApiImplicitParam(name = "orderByColumn", value = "排序列", required = false),
            @ApiImplicitParam(name = "isAsc", value = "排序的方向", required = false)
    })
    public R getOrgList(String id,String orgName,String instId,Integer pageSize,Integer pageNum,String orderByColumn,String isAsc)throws Exception {
        String url = "https://91iam-inst.9n1m.com/api/idm-service/open/inst/org/list";
        StringBuilder stb = new StringBuilder();
        stb.append("?");
        if(StringUtils.isNotBlank(id)){
            stb.append("id="+id);
        }
        if(StringUtils.isNotBlank(orgName)){
            stb.append("&");
            stb.append("orgName="+orgName);
        }
        if(StringUtils.isNotBlank(instId)){
            stb.append("&");
            stb.append("instId="+instId);
        }
        if(pageSize != null){
            stb.append("&");
            stb.append("pageSize="+pageSize);
        }
        if(pageNum != null){
            stb.append("&");
            stb.append("pageNum="+pageNum);
        }
        if(StringUtils.isNotBlank(orderByColumn)){
            stb.append("&");
            stb.append("orderByColumn="+orderByColumn);
        }
        if(StringUtils.isNotBlank(isAsc)){
            stb.append("&");
            stb.append("isAsc="+isAsc);
        }
        String urlStr = url + stb;
        //String str = HttpUtil.get(urlStr);
        URL obj = new URL(urlStr);
        HttpURLConnection con = (HttpURLConnection) obj.openConnection();

        // 添加请求头
        //con.setRequestMethod("GET");
        //con.setRequestProperty("Authorization", "eyJhbGciOiJIUzUxMiJ9.eyJzdWIiOiJCSTEwMDEiLCJpbnN0SWQiOiIxNzQ3Nzg5ODU1MjgwMTg5NTM5Iiwia2lkIjoibXhrX2F1dGhfandrIiwiaXNzIjoiaHR0cHM6Ly9xZXZvYy5jb20vIiwic3R5bGUiOiJ3ZWIiLCJleHAiOjE3MTI5MjkyMTEsImlhdCI6MTcxMjU2OTIxMSwidXNlcklkIjoiMTc3MjUzMTcxMDcyNDQ5NzQxMCIsImp0aSI6Ijk3NDc2Nzc2NzI4ODgwNzQyNCJ9.oYgjevMcC-jD4Pt0-Xs55GeOT_3vZxZRgb_HQktsFtGGW7RMA-v5Q9F7edEoF0fMbqrdKtKnF56JSgJI18ud9g");

        // 发送请求
        int responseCode = con.getResponseCode();
        log.info("Response Code : {}",responseCode);
        // 读取响应
        BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
        String inputLine;
        StringBuilder response = new StringBuilder();
        while ((inputLine = in.readLine()) != null) {
            response.append(inputLine);
        }
        in.close();
        JSONObject jsonObject = JSONObject.parseObject(response.toString());
        return R.data(jsonObject);
        //return R.data(JSONObject.parseObject(str));
    }


    /**
     * 获取用户接口地址
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/list_6
     * @param id
     * @param instId
     * @param pageSize
     * @param pageNum
     * @param orderByColumn
     * @param username
     * @param displayName
     * @param userType
     * @param departmentId
     * @param mobile
     * @param isAsc
     * @return
     * @throws Exception
     */
    @GetMapping("/getBumenList")
    @ApiOperation(value = "获取iam租户下用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = false),
            @ApiImplicitParam(name = "instId", value = "租户ID", required = false),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = false),
            @ApiImplicitParam(name = "pageNum", value = "当前页数", required = false),
            @ApiImplicitParam(name = "orderByColumn", value = "排序列", required = false),
            @ApiImplicitParam(name = "username", value = "用户账号",  required = false),
            @ApiImplicitParam(name = "displayName", value = "姓名",required = false),
            @ApiImplicitParam(name = "userType", value = "用户类型",  required = false),
            @ApiImplicitParam(name = "departmentId", value = "部门id",  required = false),
            @ApiImplicitParam(name = "mobile", value = "手机",  required = false),
            @ApiImplicitParam(name = "isAsc", value = "排序的方向", required = false)
    })
    public R getbumenList(String id,String instId,Integer pageSize,Integer pageNum,String orderByColumn,String username,String displayName,String userType,String departmentId,String mobile,String isAsc)throws Exception {
        String url = "https://91iam-inst.9n1m.com/api/users-service/open/inst/user/list";
        StringBuilder stb = new StringBuilder();
        stb.append("?");
        if(StringUtils.isNotBlank(id)){
            stb.append("id="+id);
        }
        if(StringUtils.isNotBlank(username)){
            stb.append("&");
            stb.append("username="+username);
        }
        if(StringUtils.isNotBlank(instId)){
            stb.append("&");
            stb.append("instId="+instId);
        }
        if(StringUtils.isNotBlank(displayName)){
            stb.append("&");
            stb.append("displayName="+displayName);
        }
        if(StringUtils.isNotBlank(userType)){
            stb.append("&");
            stb.append("userType="+userType);
        }
        if(StringUtils.isNotBlank(departmentId)){
            stb.append("&");
            stb.append("departmentId="+departmentId);
        }
        if(StringUtils.isNotBlank(mobile)){
            stb.append("&");
            stb.append("mobile="+mobile);
        }
        if(pageSize != null){
            stb.append("&");
            stb.append("pageSize="+pageSize);
        }
        if(pageNum != null){
            stb.append("&");
            stb.append("pageNum="+pageNum);
        }
        if(StringUtils.isNotBlank(orderByColumn)){
            stb.append("&");
            stb.append("orderByColumn="+orderByColumn);
        }
        if(StringUtils.isNotBlank(isAsc)){
            stb.append("&");
            stb.append("isAsc="+isAsc);
        }
        String urlStr = url + stb;
        URL obj = new URL(urlStr);
        HttpURLConnection con = (HttpURLConnection) obj.openConnection();
        // 发送请求
        int responseCode = con.getResponseCode();
        log.info("Response Code : {}",responseCode);
        // 读取响应
        BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
        String inputLine;
        StringBuilder response = new StringBuilder();
        while ((inputLine = in.readLine()) != null) {
            response.append(inputLine);
        }
        in.close();
        JSONObject jsonObject = JSONObject.parseObject(response.toString());
        return R.data(jsonObject);
    }


    /**
     * 开放应用角色获取：
     * 	https://91iam-inst.9n1m.com/api/doc.html#/%E6%9D%83%E9%99%90%E7%B3%BB%E7%BB%9F%E6%9C%8D%E5%8A%A1/API%E5%BC%80%E6%94%BE%E5%BA%94%E7%94%A8-%E5%B9%B3%E5%8F%B0%E8%A7%92%E8%89%B2%E5%85%B3%E8%81%94/list
     * @param appId
     * @return
     * @throws Exception
     */
    @GetMapping("/getRoleList")
    @ApiOperation(value = "获取iam开放应用角色")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "appId", value = "", required = true)
    })
    public R getRoleList(String appId)throws Exception {
        String url = "https://91iam-inst.9n1m.com/api/inst-authority-service/open/openapp/sscrole/list";
        StringBuilder stb = new StringBuilder();
        if(StringUtils.isNotBlank(appId)){
            stb.append("/"+appId);
        }
        String urlStr = url + stb;
        String str = HttpUtil.post(urlStr,"");
        log.info("str = {}",str);
        JSONObject jsonObject = JSONObject.parseObject(str);
        return R.data(jsonObject);
    }


    /**
     * 获取租户列表接口
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/queryInstInfoByInstName
     * @param instName
     * @param pageSize
     * @param pageNum
     * @param orderByColumn
     * @param isAsc
     * @return
     * @throws Exception
     */
    @GetMapping("/getInstUserList")
    @ApiOperation(value = "获取租户列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "instName", value = "企业名称", required = true),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = false),
            @ApiImplicitParam(name = "pageNum", value = "当前页数", required = false),
            @ApiImplicitParam(name = "orderByColumn", value = "排序列", required = false),
            @ApiImplicitParam(name = "isAsc", value = "排序的方向", required = false)
    })
    public R getInstUserList(String instName,Integer pageSize,Integer pageNum,String orderByColumn,String isAsc)throws Exception {
        String url = "https://91iam-inst.9n1m.com/api/users-service/open/inst/user/pageInstName";
        StringBuilder stb = new StringBuilder();
        stb.append("?");
        if(StringUtils.isNotBlank(instName)){
            stb.append("instName="+instName);
        }
        if(pageSize != null){
            stb.append("&");
            stb.append("pageSize="+pageSize);
        }
        if(pageNum != null){
            stb.append("&");
            stb.append("pageNum="+pageNum);
        }
        if(StringUtils.isNotBlank(orderByColumn)){
            stb.append("&");
            stb.append("orderByColumn="+orderByColumn);
        }
        if(StringUtils.isNotBlank(isAsc)){
            stb.append("&");
            stb.append("isAsc="+isAsc);
        }
        String urlStr = url + stb;
        //String str = HttpUtil.get(urlStr);
        URL obj = new URL(urlStr);
        HttpURLConnection con = (HttpURLConnection) obj.openConnection();

        // 发送请求
        int responseCode = con.getResponseCode();
        log.info("Response Code : {}",responseCode);
        // 读取响应
        BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
        String inputLine;
        StringBuilder response = new StringBuilder();
        while ((inputLine = in.readLine()) != null) {
            response.append(inputLine);
        }
        in.close();
        JSONObject jsonObject = JSONObject.parseObject(response.toString());
        return R.data(jsonObject);
    }




}
