package com.jnym.mgmt.client.service;

import com.jnym.mgmt.client.configs.IamProperties;
import com.jnym.mgmt.client.mapper.DelAccountMapper;
import com.jnym.mgmt.client.vo.DelAccountCheckVO;
import com.jnym.mgmt.client.vo.IamUserInstModel;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.utils.Base62Util;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

@Service
public class DelAccountService {

    @Autowired
    private DelAccountMapper delAccountMapper;
    @Autowired
    private IamInterfaceUtil iamInterfaceUtil;

    /**
     * 校验用户是否有对外分享的数据
     * type 0是注销身份校验 1是注销账号校验
     * 注销账号的时候 只需要返回个人租户的
     */
    public DelAccountCheckVO delAccountCheck(Integer type) {
        String tenantId = AuthUtil.getTenantId();
        Long instId = AuthUtil.getInstId();
        if (type==1){
            List<IamUserInstModel> iamUserInstModels = iamInterfaceUtil.userInst(AuthUtil.getUserId());
            Optional<IamUserInstModel> userInstModel = iamUserInstModels.stream().filter(iamUserInstModel -> iamUserInstModel.getInstType() == 1).findFirst();
            if (userInstModel.isPresent()){
                tenantId = Base62Util.encode(userInstModel.get().getInstId());
            }
        }
        Long userId = AuthUtil.getUserId();
        //查找分享的数据
        Integer visualLoginFreeLinkCount = delAccountMapper.visualLoginFreeLinkCount(userId, tenantId);
        Integer visualDataShareCount = delAccountMapper.visualDataShareCount(userId, tenantId);
        //封装返回值
        DelAccountCheckVO delAccountCheckVO = new DelAccountCheckVO();
        DelAccountCheckVO.DelAccountCheckData delAccountCheckData = new DelAccountCheckVO.DelAccountCheckData();
        delAccountCheckData.setCanRemove(visualLoginFreeLinkCount==0&&visualDataShareCount==0);
        List<String> mustRemove = new ArrayList<>();
        if (visualLoginFreeLinkCount>0){
            mustRemove.add(String.format("第三方web嵌入(%s)", visualLoginFreeLinkCount));
        }
        if (visualDataShareCount>0){
            mustRemove.add(String.format("大屏/报表分享(%s)", visualDataShareCount));
        }
        List<String> shouldRemove = Arrays.asList("表单录入","ETL","数据预警");
        delAccountCheckData
                .setAppId(iamInterfaceUtil.iamProperties.getAppid())
                .setAppName(iamInterfaceUtil.iamProperties.getAppName())
                .setMustRemove(mustRemove)
                .setShouldRemove(shouldRemove)
        ;
        return delAccountCheckVO
                .setData(delAccountCheckData)
                .setSuccess(true)
                .setCode(200)
                .setMessage("ok")
                ;
    }

    /**
     * 注销账户
     * 查找个人租户 删除数据即可
     *
     * @return
     */
    public Boolean delAccount() {
        //获取个人租户
        List<IamUserInstModel> iamUserInstModels = iamInterfaceUtil.userInst(AuthUtil.getUserId());
        Optional<IamUserInstModel> userInstModel = iamUserInstModels.stream().filter(iamUserInstModel -> iamUserInstModel.getInstType() == 1).findFirst();
        if (userInstModel.isPresent()) {
            String tenantId = Base62Util.encode(userInstModel.get().getInstId());
            delAccountMapper.delDs(null,tenantId);
            delAccountMapper.delCubeData(null,tenantId);
            delAccountMapper.delVisualData(null,tenantId);
        }
        return true;
    }

    /**
     * 注销身份 只能注销企业身份（超管）
     * 只有在管理员的情况下删除租户数据
     */
    public Boolean delIdentity(){
        if (AuthUtil.isAdmin()){
            delAccountMapper.delDs(null,AuthUtil.getTenantId());
            delAccountMapper.delCubeData(null,AuthUtil.getTenantId());
            delAccountMapper.delVisualData(null,AuthUtil.getTenantId());
        }
        return true;
    }


}
