package com.jnym.mgmt.client.service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jnym.mgmt.client.entity.DeveloperApply;
import com.jnym.mgmt.client.entity.DeveloperReviewLog;
import com.jnym.mgmt.client.mapper.DeveloperApplyMapper;
import com.jnym.mgmt.client.vo.DeveloperApplyPageParams;
import com.jnym.mgmt.client.vo.DeveloperApplyReviewParams;
import com.jnym.mgmt.client.vo.DeveloperApplyStatusCountVO;
import com.jnym.mgmt.client.vo.IamSetRoleParams;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.utils.Base62Util;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.dto.IamGetUserByIdsParams;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class DeveloperApplyManageService {

    @Autowired
    private DeveloperReviewLogService developerReviewLogService;
    @Autowired
    private IamService iamService;
    @Autowired
    private IamInterfaceUtil iamInterfaceUtil;
    @Autowired
    private DeveloperApplyService developerApplyService;

    public Page<DeveloperApply> applyPage(DeveloperApplyPageParams params) {
        Page<DeveloperApply> page = developerApplyService.lambdaQuery()
                .eq(params.getUserId() != null, DeveloperApply::getUserId, params.getUserId())
                .eq(params.getStatus() != null, DeveloperApply::getStatus, params.getStatus())
                .orderByDesc(DeveloperApply::getApplyTime)
                .page(new Page<>(params.getCurrent(), params.getSize()));
        if (page.getRecords().size() != 0) {
            //租户信息
            List<String> tenantIds = page.getRecords().stream().map(DeveloperApply::getTenantId).distinct().collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = iamService.getTenantNames(tenantIds);
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            //用户信息
            List<Long> userIds = page.getRecords().stream().map(DeveloperApply::getCreateUserId).collect(Collectors.toList());
            List<UserModel> userModelList = iamService.getByUserIds(new IamGetUserByIdsParams(userIds, null, 0));
            Map<Long, UserModel> accountMap = userModelList.stream().collect(Collectors.toMap(UserModel::getId, Function.identity()));
            for (DeveloperApply record : page.getRecords()) {
                String s = tenantMap.get(record.getTenantId());
                record.setTenantName(s);
                UserModel userModel = accountMap.get(record.getCreateUserId());
                if (userModel != null) {
                    record.setUserNickName(userModel.getNickName())
                            .setUserAccount(userModel.getAccount());
                }
            }
        }
        return page;
    }

    /**
     * 各状态数据个数
     */
    public DeveloperApplyStatusCountVO statusCount() {
        List<DeveloperApply> list = developerApplyService.lambdaQuery()
                .list();

        Map<Integer, Long> statusCountMap = list.stream()
                .collect(Collectors.groupingBy(DeveloperApply::getStatus, Collectors.counting()));

        DeveloperApplyStatusCountVO developerApplyStatusCountVO = new DeveloperApplyStatusCountVO();

        // 设置统计数据
        developerApplyStatusCountVO.setAll(Long.valueOf(list.size()))
                .setWithdraw(statusCountMap.getOrDefault(0, 0L))
                .setNotReview(statusCountMap.getOrDefault(1, 0L))
                .setReviewed(statusCountMap.getOrDefault(2, 0L))
                .setReject(statusCountMap.getOrDefault(3, 0L));

        return developerApplyStatusCountVO;
    }

    /**
     * 审核记录
     *
     * @param id
     * @return
     */
    public List<DeveloperReviewLog> reviewLogList(Long id) {
        List<DeveloperReviewLog> list = developerReviewLogService.lambdaQuery()
                .eq(DeveloperReviewLog::getRelationId, id)
                .orderByDesc(DeveloperReviewLog::getCreateTime)
                .list();
        return list;
    }

    @Transactional(rollbackFor = Exception.class)
    public Boolean review(DeveloperApplyReviewParams params) {
        for (Long id : params.getIds()) {
            DeveloperApply developerApply = developerApplyService.lambdaQuery()
                    .eq(DeveloperApply::getId, id)
                    .one();
            if (developerApply == null) {
                throw new RuntimeException("id不存在");
            }
            developerApply.setStatus(params.isPass() ? 2 : 3);
            developerApply.setCheckTime(new Date());
            developerApplyService.updateById(developerApply);
            DeveloperReviewLog visualReviewLog = new DeveloperReviewLog();
            visualReviewLog
                    .setRelationId(id)
                    .setRemark(params.getReason())
                    .setStatus(params.isPass() ? 2 : 3)
                    .setCreateUserName(SecureUtil.getNickName())
                    .setCreateTime(LocalDateTime.now())
                    .setCreateUserId(SecureUtil.getUserId());
            developerReviewLogService.save(visualReviewLog);
            // 如果是通过的话 调用接口分配角色
            if (params.isPass()) {
                IamSetRoleParams iamSetRoleParams = new IamSetRoleParams();
                iamSetRoleParams.setUserId(developerApply.getUserId())
                        .setInstId(Base62Util.decode(developerApply.getTenantId()))
                        .setRoleIds(iamInterfaceUtil.iamProperties.getDevRoleIds());
                iamInterfaceUtil.setRole(iamSetRoleParams);
            }
        }
        return true;
    }

}


