package com.jnym.mgmt.client.service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jnym.mgmt.client.entity.CustomPlatform;
import com.jnym.mgmt.client.entity.DeveloperApply;
import com.jnym.mgmt.client.entity.DeveloperReviewLog;
import com.jnym.mgmt.client.mapper.CustomPlatformMapper;
import com.jnym.mgmt.client.mapper.DeveloperApplyMapper;
import org.springblade.core.secure.utils.AuthUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

@Service
public class DeveloperApplyService extends ServiceImpl<DeveloperApplyMapper, DeveloperApply> implements IService<DeveloperApply> {

    @Autowired
    private DeveloperReviewLogService developerReviewLogService;

    @Transactional(rollbackFor = Exception.class)
    public Boolean apply() {
        Long userId = AuthUtil.getUserId();
        //添加申请
        DeveloperApply developerApply = this.getOne(Wrappers.<DeveloperApply>lambdaQuery().eq(DeveloperApply::getUserId, userId));
        if (developerApply == null) {
            developerApply = new DeveloperApply();
            developerApply
                    .setInstType((Integer) AuthUtil.getDetail().get("instType"))
                    .setTenantId(AuthUtil.getTenantId())
                    .setUserId(userId)
                    .setCreateUserId(userId)
                    .setCreateTime(new Date())
                    .setApplyTime(new Date())
                    .setStatus(1);
        } else {
            developerApply
                    .setApplyTime(new Date())
                    .setStatus(1);
        }
        this.saveOrUpdate(developerApply);
        //添加记录日志
        DeveloperReviewLog developerReviewLog = new DeveloperReviewLog();
        developerReviewLog.setTenantId(AuthUtil.getTenantId());
        developerReviewLog
                .setCreateTime(LocalDateTime.now())
                .setRelationId(developerApply.getId())
                .setCreateUserName(AuthUtil.getNickName())
                .setStatus(1)
                .setCreateUserId(userId);
        return developerReviewLogService.save(developerReviewLog);
    }

    public DeveloperApply currentUser() {
        DeveloperApply developerApply = this.getOne(Wrappers.<DeveloperApply>lambdaQuery().eq(DeveloperApply::getUserId, AuthUtil.getUserId()));
        if (developerApply != null) {
            List<DeveloperReviewLog> list = developerReviewLogService.getBaseMapper().findByRelationId(developerApply.getId());
            developerApply.setDeveloperReviewLogs(list);
        }
        return developerApply;
    }

    public boolean withdraw() {
        DeveloperApply developerApply = this.getOne(Wrappers.<DeveloperApply>lambdaQuery().eq(DeveloperApply::getUserId, AuthUtil.getUserId()));
        developerApply.setStatus(0);
        this.saveOrUpdate(developerApply);
        //添加记录日志
        DeveloperReviewLog developerReviewLog = new DeveloperReviewLog();
        developerReviewLog.setTenantId(AuthUtil.getTenantId());
        developerReviewLog
                .setCreateTime(LocalDateTime.now())
                .setRelationId(developerApply.getId())
                .setCreateUserName(AuthUtil.getNickName())
                .setStatus(4)
                .setCreateUserId(AuthUtil.getUserId());
        return developerReviewLogService.save(developerReviewLog);
    }

}


