package com.jnym.mgmt.client.service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.jnym.mgmt.client.configs.IamProperties;
import com.jnym.mgmt.client.entity.IamInstUserInfoPO;
import com.jnym.mgmt.client.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.utils.Base62Util;
import org.springblade.system.client.dto.IamGetUserByIdsParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class IamInterfaceUtil {

    @Autowired
    public IamProperties iamProperties;

    private RestTemplate restTemplate = new RestTemplate();

    /**
     * 组织树
     * https://91iam-inst.9n1m.com/api/doc.html#/%E8%BA%AB%E4%BB%BD%E7%AE%A1%E7%90%86%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%BB%84%E7%BB%87%E7%AE%A1%E7%90%86/tree_2
     */
    public List<IamOrgModel> orgTree(long instId) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() + "idm-service/open/inst/org/tree/{instId}", IamResponse.class, instId);
        if (!checkResponse(response)) {
            throw new RuntimeException("iam-通过部门获取用户 接口请求失败--" + response);
        }
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamOrgModel> iamOrgModels = JSON.parseArray(jsonString, IamOrgModel.class);
            return iamOrgModels;
        }
        return Collections.emptyList();
    }

    /**
     * 通过组织获取用户
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/queryListByDepartment
     */
    public List<IamInstUserInfoPO> getUserByDept(long departmentId) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() + "users-service/open/inst/user/getByDepartment/{departmentId}", IamResponse.class, departmentId);
        if (!checkResponse(response)) {
            throw new RuntimeException("iam-通过部门获取用户 接口请求失败--" + response);
        }
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamInstUserInfoPO> iamInstUserInfoPOS = JSON.parseArray(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPOS;
        }
        return Collections.emptyList();
    }


    /**
     * 通过角色获取用户列表
     * https://91iam-inst.9n1m.com/api/doc.html#/%E6%9D%83%E9%99%90%E7%B3%BB%E7%BB%9F%E6%9C%8D%E5%8A%A1/API%EF%BC%88%E5%86%85%E9%83%A8%E6%9C%8D%E5%8A%A1%E8%B0%83%E7%94%A8%EF%BC%89-%E8%A7%92%E8%89%B2-%E7%94%A8%E6%88%B7%E5%85%B3%E8%81%94/listUser
     */
    public List<IamInstUserInfoPO> getUserByRole(long roleId, Long instId) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() + "inst-authority-service/inner/inst/role/user/list?roleId={roleId}&instId={instId}", IamResponse.class, roleId, instId);
        if (!checkResponse(response)) {
            throw new RuntimeException("iam-通过角色获取用户 接口请求失败--" + response);
        }
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamInstUserInfoPO> iamInstUserInfoPOS = JSON.parseArray(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPOS;
        }
        return Collections.emptyList();
    }


    /**
     * 分页查询租户用户
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/list_6
     */
    public GetIamUserPageModel userPage(GetIamUserPageParams params) {

        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "users-service/open/inst/user/list?userId={userId}&username={username}&displayName={displayName}&instId={instId}&userType={userType}&departmentId={departmentId}&mobile={mobile}&pageSize={pageSize}&pageNum={pageNum}&orderByColumn={orderByColumn}&isAsc={isAsc}", IamResponse.class,
                params.getUserId(), params.getUsername(), params.getDisplayName(), params.getInstId(), params.getUserType(), params.getDepartmentId(), params.getMobile(), params.getPageSize(), params.getPageNum(), params.getOrderByColumn(), params.getIsAsc());
        if (!checkResponse(response))
            throw new RuntimeException("iam-分页查询租户用户 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            GetIamUserPageModel iamUserPageModel = JSON.parseObject(jsonString, GetIamUserPageModel.class);
            return iamUserPageModel;
        }
        return null;
    }

    /**
     * 根据用户id查询用户信息
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/getUserBriefInfo
     *
     * @param userId
     * @return
     */
    public IamInstUserInfoPO getByUserId(Long userId, long instId) {
        log.info("iam-根据用户id查询用户信息 请求参数 userid={},instId={}", userId, instId);
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                "users-service/open/inst/user/getUserBriefInfo?instId={instId}&userId={userId}", IamResponse.class, instId, userId);
        if (!checkResponse(response)) {
            throw new RuntimeException("iam-根据用户id查询用户信息 接口请求失败--" + response);
        }
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            IamInstUserInfoPO iamInstUserInfoPO = JSON.parseObject(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPO;
        }
        return null;
    }


    /**
     * 根据用户ids查询用户信息-非明文
     *
     * @param params
     * @return
     */
    public List<IamInstUserInfoPO> getByUserIds1(IamGetUserByIdsParams params) {
        ResponseEntity<IamResponse> response = restTemplate.postForEntity(iamProperties.getUrl() +
                "users-service/open/inst/user/getUserListByUserIds", params, IamResponse.class);
        if (!checkResponse(response)) {
            throw new RuntimeException("iam-根据用户ids查询用户信息-非明文 接口请求失败--" + response);
        }
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamInstUserInfoPO> iamInstUserInfoPOS = JSON.parseArray(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPOS;
        }
        return Collections.emptyList();
    }


    /**
     * 根据昵称或者用户名查询用户信息
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/getUserByNickName
     *
     * @param params
     * @return
     */
    public GetIamUserPageModel getUserByAccountAndNickName(GetUserByAccountAndNickNameParams params) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "users-service/open/inst/user/getUserByNickName?instId={instId}&keyName={keyName}&pageSize={pageSize}&pageNum={pageNum}", IamResponse.class,
                params.getInstId(), params.getKeyName(), params.getPageSize(), params.getPageNum());
        if (!checkResponse(response))
            throw new RuntimeException("iam-根据昵称或者用户名查询用户信息 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            GetIamUserPageModel iamUserPageModel = JSON.parseObject(jsonString, GetIamUserPageModel.class);
            return iamUserPageModel;
        }
        return null;
    }

    /**
     * 根据用户ids查询用户信息-明文
     *
     * @param params
     * @return
     */
    public List<IamInstUserInfoPO> getByUserIds2(IamGetUserByIdsParams params) {
        ResponseEntity<IamResponse> response = restTemplate.postForEntity(iamProperties.getUrl() +
                "users-service/inner/userInst/listByUserIdsPlaintext", params, IamResponse.class);
        if (!checkResponse(response)) {
            throw new RuntimeException("iam-根据用户ids查询用户信息-明文 接口请求失败--" + response);
        }
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamInstUserInfoPO> iamInstUserInfoPOS = JSON.parseArray(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPOS;
        }
        return Collections.emptyList();
    }

    /**
     * 根据账号或者手机号精确查询用户
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/getUserByUsernameOrPhone
     */
    public List<IamInstUserInfoPO> getUserByUsernameOrPhone(Long instId, String keyName) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "users-service/open/inst/user/getUserByUsernameOrPhone?instId={instId}&keyName={keyName}", IamResponse.class,
                instId, keyName);
        if (!checkResponse(response))
            throw new RuntimeException("iam-根据账号或者手机号精确查询用户 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamInstUserInfoPO> iamInstUserInfoPOS = JSON.parseArray(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPOS;
        }
        return Collections.emptyList();
    }

    /**
     * 查询某组织下的用户
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/listOrgRelatedVo
     */
    public GetIamUserPageModel getOrgRelatedUser(GetIamOrgUserParams params) {

        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "users-service/open/inst/user/getOrgRelatedUser?instId={instId}&departmentId={departmentId}&keyName={keyName}&pageSize={pageSize}&pageNum={pageNum}&orderByColumn={orderByColumn}&isAsc={isAsc}", IamResponse.class,
                params.getInstId(), params.getDepartmentId(), params.getKeyName(), params.getPageSize(), params.getPageNum(), params.getOrderByColumn(), params.getIsAsc());
        if (!checkResponse(response))
            throw new RuntimeException("iam-查询某组织下的用户 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            GetIamUserPageModel iamUserPageModel = JSON.parseObject(jsonString, GetIamUserPageModel.class);
            return iamUserPageModel;
        }
        return null;
    }

    /**
     * 通过租户ids获取租户名字
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/getInstNamesInstIds
     *
     * @return
     */
    public List<IamInstIdNameModel> getTenantNames(GetInstNamesParams params) {
        ResponseEntity<IamResponse> response = restTemplate.postForEntity(iamProperties.getUrl() + "users-service/inner/userInst/listInstName", params, IamResponse.class);
        if (!checkResponse(response))
            throw new RuntimeException("iam-通过租户ids获取租户名字 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamInstIdNameModel> iamInstIdNameModels = JSON.parseArray(jsonString, IamInstIdNameModel.class);
            return iamInstIdNameModels;
        }
        return Collections.emptyList();
    }


    /**
     * 通过名字获取租户信息
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/queryInstInfoByInstName
     *
     * @param params
     * @return
     */
    public GetIamTenantListModel getTenantList(GetTenantListParams params) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "users-service/open/inst/user/pageInstName?instName={instName}&pageSize={pageSize}&pageNum={pageNum}&orderByColumn={orderByColumn}&isAsc={isAsc}",
                IamResponse.class, params.getInstName(), params.getPageSize(), params.getPageNum(), params.getOrderByColumn(), params.getIsAsc());
        if (!checkResponse(response))
            throw new RuntimeException("iam-通过名字获取租户信息 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            GetIamTenantListModel getIamTenantListModel = JSON.parseObject(jsonString, GetIamTenantListModel.class);
            return getIamTenantListModel;
        }
        return null;
    }

    /**
     * 获取平台角色信息
     * https://91iam-inst.9n1m.com/api/doc.html#/%E6%9D%83%E9%99%90%E7%B3%BB%E7%BB%9F%E6%9C%8D%E5%8A%A1/API%E6%9D%83%E9%99%90%E5%B9%B3%E5%8F%B0%E7%B3%BB%E7%BB%9F%E7%A7%9F%E6%88%B7%E8%A7%92%E8%89%B2/instPlatformRoles
     */
    public GetIamRoleListModel getRoleList(GetIamRoleListParams params) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "inst-authority-service/open/inst/sys/platform-role-page?instId={instId}&roleName={roleName}&pageSize={pageSize}&pageNum={pageNum}&orderByColumn={orderByColumn}&isAsc={isAsc}",
                IamResponse.class, params.getInstId(), params.getRoleName(), params.getPageSize(), params.getPageNum(), params.getOrderByColumn(), params.getIsAsc());
        if (!checkResponse(response))
            throw new RuntimeException("iam-获取平台角色信息 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            GetIamRoleListModel getIamRoleListModel = JSON.parseObject(jsonString, GetIamRoleListModel.class);
            return getIamRoleListModel;
        }
        return null;
    }

    /**
     * 分页查询租户信息
     */
    public GetIamTenantListModel tenantPage(String name) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "users-service/open/inst/user/listEnterpriseByName?instName={name}",
                IamResponse.class, name);
        if (!checkResponse(response))
            throw new RuntimeException("iam-分页查询租户信息 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            GetIamTenantListModel getIamTenantListModel = JSON.parseObject(jsonString, GetIamTenantListModel.class);
            return getIamTenantListModel;
        }
        return null;
    }

    public static void main(String[] args) {
        long decode = Base62Util.decode("29MtpCk8TQd");
        System.out.println(decode);
    }

    /**
     * 根据账号精确查询用户(不包含子用户)
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/exactSearchUser
     */
    public IamInstUserInfoPO exactUser(Long instId, String username) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "users-service/open/inst/user/exactUser?instId={instId}&username={username}", IamResponse.class,
                instId, username);
        if (!checkResponse(response))
            return null;
        //throw new RuntimeException("iam-根据账号精确查询用户(不包含子用户) 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            IamInstUserInfoPO iamInstUserInfoPO = JSON.parseObject(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPO;
        }
        return null;
    }


    /**
     * 根据账号、手机或者邮箱精确查询用户(不包含子用户)
     * https://91iam-inst.9n1m.com/api/doc.html#/%E7%94%A8%E6%88%B7%E6%9C%8D%E5%8A%A1/API%E7%A7%9F%E6%88%B7%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/exactSearchUserByCommonInfo
     */
    public IamInstUserInfoPO exactUserCommon(String keyName) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                        "users-service/open/inst/user/exactUser/common?keyName={keyName}", IamResponse.class,
                keyName);
        if (!checkResponse(response))
            return null;
        //            throw new RuntimeException("iam-根据账号、手机或者邮箱精确查询用户(不包含子用户) 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            IamInstUserInfoPO iamInstUserInfoPO = JSON.parseObject(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPO;
        }
        return null;
    }

    public List<IamUserInstModel> userInst(Long userId) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getUrl() +
                "users-service/open/inst/user/inst/" + userId, IamResponse.class);
        if (!checkResponse(response))
            return null;
        //            throw new RuntimeException("iam-根据账号、手机或者邮箱精确查询用户(不包含子用户) 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamUserInstModel> iamUserInstModels = JSON.parseArray(jsonString, IamUserInstModel.class);
            return iamUserInstModels;
        }
        return Collections.emptyList();
    }

    /**
     * 开发者申请成功 给用户设置角色
     * https://91iam-inst.9n1m.com/api/doc.html#/%E6%9D%83%E9%99%90%E7%B3%BB%E7%BB%9F%E6%9C%8D%E5%8A%A1/API%E6%9D%83%E9%99%90%E7%B3%BB%E7%BB%9F-%E5%B9%B3%E5%8F%B0%E8%A7%92%E8%89%B2-%E7%94%A8%E6%88%B7%E5%85%B3%E8%81%94/update_15
     *
     * @return
     */
    public Boolean setRole(IamSetRoleParams params) {
        ResponseEntity<IamResponse> response = restTemplate
                .postForEntity(iamProperties.getUrl() + "inst-authority-service/open/inst/ssc/role/users/subject/update", params, IamResponse.class);
        if (!checkResponse(response))
            throw new RuntimeException("iam-开发者申请成功 给用户设置角色 接口请求失败--" + response);
        if (response.getBody().isSuccess() && response.getBody().getCode() == 0) {
            return true;
        }
        return false;
    }


    /**
     * 对外接口-根据账号或者名称精确查询用户信息
     * https://91iam-mgmt.9n1m.com/api/doc.html#/%E5%B9%B3%E5%8F%B0%E8%BA%AB%E4%BB%BD%E7%AE%A1%E7%90%86/API%E7%94%A8%E6%88%B7%E7%AE%A1%E7%90%86/listByKeyName
     *
     * @return
     */
    public List<IamInstUserInfoPO> manageListByKeyName(String keyName) {
        ResponseEntity<IamResponse> response = restTemplate.getForEntity(iamProperties.getManageUrl() +
                        "mgmt-idm-service/open/user/query/listByKeyName/" + keyName, IamResponse.class,
                keyName);
        if (!checkResponse(response))
            throw new RuntimeException("iam-管理端-根据账号或者名称精确查询用户信息 接口请求失败--" + response);
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<IamInstUserInfoPO> iamInstUserInfoPOS = JSON.parseArray(jsonString, IamInstUserInfoPO.class);
            return iamInstUserInfoPOS;
        }
        return Collections.emptyList();
    }

    private boolean checkResponse(ResponseEntity<IamResponse> response) {
        if (!response.getStatusCode().is2xxSuccessful() || !response.getBody().isSuccess()) {
            return false;
        }
        return true;
    }

}
