package com.jnym.mgmt.client.service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jnym.mgmt.client.vo.*;
import com.jnym.mgmt.client.entity.IamInstUserInfoPO;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.node.ForestNodeMerger;
import org.springblade.core.tool.node.TreeNode;
import org.springblade.core.tool.utils.Base62Util;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.modules.system.entity.Tenant;
import org.springblade.modules.system.vo.UserVO;
import org.springblade.system.client.dto.IamGetUserByIdsParams;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Slf4j
@Service
public class IamService {

    @Autowired
    private IamInterfaceUtil iamInterfaceUtil;

    public List<TreeNode> orgTree() {
        List<IamOrgModel> orgTree = iamInterfaceUtil.orgTree(AuthUtil.getInstId());
        if (CollectionUtil.isNotEmpty(orgTree)) {
            //实体转换
            ArrayList<TreeNode> treeNodes = new ArrayList<>();
            convertData(orgTree, treeNodes);
            List<TreeNode> merge = ForestNodeMerger.merge(treeNodes);
            return merge;
        }
        return Collections.emptyList();
    }

    private void convertData(List<IamOrgModel> orgTree, List<TreeNode> treeNodes) {
        for (IamOrgModel iamOrgModel : orgTree) {
            TreeNode treeNode = new TreeNode();
            treeNodes.add(treeNode);
            treeNode.setId(iamOrgModel.getId());
            treeNode.setParentId(iamOrgModel.getParentId());
            treeNode.setKey(iamOrgModel.getId());
            treeNode.setValue(iamOrgModel.getId());
            treeNode.setTitle(iamOrgModel.getName());
            treeNode.setHasChildren(false);
            if (CollectionUtil.isNotEmpty(iamOrgModel.getChildren())) {
                treeNode.setHasChildren(true);
                convertData(iamOrgModel.getChildren(), treeNodes);
            }
        }
    }


    public List<UserVO> getUserByDept(long departmentId) {
        List<IamInstUserInfoPO> iamInstUserInfoPOS = iamInterfaceUtil.getUserByDept(departmentId);
        ArrayList<UserVO> userVOS = new ArrayList<>();
        for (IamInstUserInfoPO iamInstUserInfoPO : iamInstUserInfoPOS) {
            UserVO userVO = new UserVO();
            userVOS.add(userVO);
            userVO.setId(Long.parseLong(iamInstUserInfoPO.getUserId()));
            userVO.setNickName(iamInstUserInfoPO.getNickName());
            userVO.setRealName(iamInstUserInfoPO.getNickName());
            userVO.setAccount(iamInstUserInfoPO.getUsername());
        }
        return userVOS;
    }


    public List<UserVO> getUserByRole(long departmentId, Long instId) {
        List<IamInstUserInfoPO> iamInstUserInfoPOS = iamInterfaceUtil.getUserByRole(departmentId, instId);
        ArrayList<UserVO> userVOS = new ArrayList<>();
        for (IamInstUserInfoPO iamInstUserInfoPO : iamInstUserInfoPOS) {
            UserVO userVO = new UserVO();
            userVOS.add(userVO);
            userVO.setId(Long.parseLong(iamInstUserInfoPO.getUserId()));
            userVO.setNickName(iamInstUserInfoPO.getNickName());
            userVO.setRealName(iamInstUserInfoPO.getNickName());
            userVO.setAccount(iamInstUserInfoPO.getUsername());
        }
        return userVOS;
    }

    public List<TenantModel> getTenantNames(List<String> tenantIds) {
        GetInstNamesParams getInstNamesParams = new GetInstNamesParams();
        ArrayList<Long> longs = new ArrayList<>();
        getInstNamesParams.setInstIds(longs);
        for (String tenantId : tenantIds) {
            try {
                longs.add(Base62Util.decode(tenantId));
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        }
        ArrayList<TenantModel> tenantModels = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(longs)) {
            List<IamInstIdNameModel> iamInstIdNameModels = iamInterfaceUtil.getTenantNames(getInstNamesParams);
            for (IamInstIdNameModel iamInstIdNameModel : iamInstIdNameModels) {
                TenantModel tenantModel = new TenantModel();
                tenantModels.add(tenantModel);
                tenantModel.setTenantId(Base62Util.encode(iamInstIdNameModel.getInstId()));
                tenantModel.setTenantName(iamInstIdNameModel.getInstName());
            }
        }
        return tenantModels;
    }

    public IPage<TenantModel> getTenantList(GetTenantListParams params) {
        GetIamTenantListModel iamTenantListModel = iamInterfaceUtil.getTenantList(params);
        ArrayList<TenantModel> tenantModels = new ArrayList<>();
        for (IamInstIdNameModel iamInstIdNameModel : iamTenantListModel.getRecords()) {
            TenantModel tenantModel = new TenantModel();
            tenantModels.add(tenantModel);
            tenantModel.setTenantId(Base62Util.encode(iamInstIdNameModel.getInstId()));
            tenantModel.setTenantName(iamInstIdNameModel.getInstName());
        }
        Page<TenantModel> tenantModelPage = new Page<>();
        tenantModelPage.setRecords(tenantModels);
        tenantModelPage.setTotal(iamTenantListModel.getTotal());
        tenantModelPage.setCurrent(iamTenantListModel.getCurrent());
        tenantModelPage.setSize(iamTenantListModel.getSize());
        tenantModelPage.setPages(iamTenantListModel.getPages());
        return tenantModelPage;
    }

    /**
     * 查询个人租户或者公司租户
     *
     * @return
     */
    public IPage<Tenant> tenantPage(TenantPageParams params) {
        Page<Tenant> tenantPage = new Page<>();
        ArrayList<Tenant> tenants = new ArrayList<>();
        if (params.getType() == 0) {
            GetIamTenantListModel getIamTenantListModel = iamInterfaceUtil.tenantPage(params.getKey());
            for (IamInstIdNameModel iamInstIdNameModel : getIamTenantListModel.getRecords()) {
                Tenant tenant = new Tenant();
                tenants.add(tenant);
                tenant.setTenantId(Base62Util.encode(iamInstIdNameModel.getInstId()));
                tenant.setTenantName(iamInstIdNameModel.getInstName());
            }
            tenantPage.setRecords(tenants);
            tenantPage.setTotal(getIamTenantListModel.getTotal());
            tenantPage.setCurrent(getIamTenantListModel.getCurrent());
            tenantPage.setSize(getIamTenantListModel.getSize());
            tenantPage.setPages(getIamTenantListModel.getPages());
        } else if (params.getType() == 1) {
            IamInstUserInfoPO iamInstUserInfoPO = iamInterfaceUtil.exactUser(null, params.getKey());
            if (iamInstUserInfoPO != null) {
                Tenant tenant = new Tenant();
                tenants.add(tenant);
                tenant.setTenantId(Base62Util.encode(iamInstUserInfoPO.getInstId()));
                tenant.setTenantName(iamInstUserInfoPO.getNickName());
                tenantPage.setRecords(tenants);
                tenantPage.setTotal(1);
                tenantPage.setCurrent(1);
                tenantPage.setSize(params.getPageSize());
                tenantPage.setPages(params.getPageNum());
            }
        }
        return tenantPage;
    }

    public Page<UserVO> userPage(GetIamUserPageParams params) {
        GetIamOrgUserParams getIamOrgUserParams = new GetIamOrgUserParams();
        getIamOrgUserParams.setInstId(AuthUtil.getInstId());
        getIamOrgUserParams.setKeyName(params.getUsername());
        getIamOrgUserParams.setPageNum(params.getPageNum());
        getIamOrgUserParams.setPageSize(params.getPageSize());
        GetIamUserPageModel iamUserPageModel = iamInterfaceUtil.getOrgRelatedUser(getIamOrgUserParams);
        ArrayList<UserVO> userVOS = new ArrayList<>();
        for (IamInstUserInfoPO iamInstUserInfoPO : iamUserPageModel.getRecords()) {
            UserVO userVO = new UserVO();
            userVOS.add(userVO);
            userVO.setId(Long.parseLong(iamInstUserInfoPO.getUserId()));
            userVO.setNickName(iamInstUserInfoPO.getNickName());
            userVO.setRealName(iamInstUserInfoPO.getNickName());
            userVO.setAccount(iamInstUserInfoPO.getUsername());
        }
        Page<UserVO> page = new Page<>();
        page.setRecords(userVOS);
        page.setTotal(iamUserPageModel.getTotal());
        page.setCurrent(iamUserPageModel.getCurrent());
        page.setSize(iamUserPageModel.getSize());
        page.setPages(iamUserPageModel.getPages());
        return page;
    }

    public List<UserVO> getUserByUsernameOrPhone(String keyName, boolean ignoreTenant) {
        List<IamInstUserInfoPO> userByUsernameOrPhone = iamInterfaceUtil.getUserByUsernameOrPhone(ignoreTenant ? null : AuthUtil.getInstId(), keyName);
        ArrayList<UserVO> userVOS = new ArrayList<>();
        for (IamInstUserInfoPO iamInstUserInfoPO : userByUsernameOrPhone) {
            UserVO userVO = new UserVO();
            userVOS.add(userVO);
            userVO.setId(Long.parseLong(iamInstUserInfoPO.getUserId()));
            userVO.setTenantId(Base62Util.encode(iamInstUserInfoPO.getInstId()));
            userVO.setNickName(iamInstUserInfoPO.getNickName());
            userVO.setRealName(iamInstUserInfoPO.getNickName());
            userVO.setAccount(iamInstUserInfoPO.getUsername());
        }
        return userVOS;
    }


    public UserVO exactUserCommon(String keyName) {
        IamInstUserInfoPO iamInstUserInfoPO = iamInterfaceUtil.exactUserCommon(keyName);
        if (iamInstUserInfoPO == null) return null;
        UserVO userVO = new UserVO();
        userVO.setId(Long.parseLong(iamInstUserInfoPO.getUserId()));
        userVO.setTenantId(Base62Util.encode(iamInstUserInfoPO.getInstId()));
        userVO.setNickName(iamInstUserInfoPO.getNickName());
        userVO.setRealName(iamInstUserInfoPO.getNickName());
        userVO.setAccount(iamInstUserInfoPO.getUsername());
        return userVO;
    }

    public List<UserVO> manageGetByKeyName(String keyName) {
        List<IamInstUserInfoPO> iamInstUserInfoPOList = iamInterfaceUtil.manageListByKeyName(keyName);
        ArrayList<UserVO> userVOS = new ArrayList<>();
        for (IamInstUserInfoPO iamInstUserInfoPO : iamInstUserInfoPOList) {
            UserVO userVO = new UserVO();
            userVOS.add(userVO);
            userVO.setId(Long.parseLong(iamInstUserInfoPO.getUserId()));
            userVO.setNickName(iamInstUserInfoPO.getNickName());
            userVO.setRealName(iamInstUserInfoPO.getNickName());
            userVO.setAccount(iamInstUserInfoPO.getUsername());
        }
        return userVOS;
    }


    public GetIamRoleListModel getRoleList(GetIamRoleListParams params) {
        params.setInstId(AuthUtil.getInstId());
        return iamInterfaceUtil.getRoleList(params);
    }

    public UserModel getByUserId(Long userId, Long tenantId) {
        IamInstUserInfoPO instUserInfoPO = iamInterfaceUtil.getByUserId(userId, tenantId);
        if (instUserInfoPO != null) {
            UserModel userModel = new UserModel();
            userModel.setUserId(instUserInfoPO.getUserId());
            userModel.setAccount(instUserInfoPO.getUsername());
            userModel.setEmail(instUserInfoPO.getEmail());
            userModel.setPhone(instUserInfoPO.getMobile());
            userModel.setNickName(instUserInfoPO.getNickName());
            userModel.setSex(instUserInfoPO.getGender());
            userModel.setIsAdmin(instUserInfoPO.getIsAdmin());
            userModel.setInstId(instUserInfoPO.getInstId());
            userModel.setInstType(instUserInfoPO.getInstType());
            userModel.setUserType(instUserInfoPO.getUserType());
            userModel.setDisplayName(instUserInfoPO.getDisplayName());
            return userModel;
        }
        return null;
    }

    public List<UserModel> getByUserIds(IamGetUserByIdsParams params) {
        List<IamInstUserInfoPO> infoPOList;
        if (params.getHide() == 0) {
            infoPOList = iamInterfaceUtil.getByUserIds1(params);
        } else {
            infoPOList = iamInterfaceUtil.getByUserIds2(params);
        }
        if (infoPOList.size() != 0) {
            List<UserModel> userModels = new ArrayList<>();
            for (IamInstUserInfoPO instUserInfoPO : infoPOList) {
                UserModel userModel = new UserModel();
                userModels.add(userModel);
                userModel.setId(Long.parseLong(instUserInfoPO.getUserId()));
                userModel.setEmail(instUserInfoPO.getEmail());
                userModel.setPhone(instUserInfoPO.getMobile());
                userModel.setNickName(instUserInfoPO.getNickName());
                userModel.setAccount(instUserInfoPO.getUsername());
                userModel.setSex(instUserInfoPO.getGender());
            }
            return userModels;
        }
        return Collections.emptyList();
    }

    public Page<UserVO> getUserByAccountAndNickName(GetUserByAccountAndNickNameParams params){
        GetIamUserPageModel iamUserPageModel = iamInterfaceUtil.getUserByAccountAndNickName(params);
        ArrayList<UserVO> userVOS = new ArrayList<>();
        for (IamInstUserInfoPO iamInstUserInfoPO : iamUserPageModel.getRecords()) {
            UserVO userVO = new UserVO();
            userVOS.add(userVO);
            userVO.setId(Long.parseLong(iamInstUserInfoPO.getUserId()));
            userVO.setNickName(iamInstUserInfoPO.getNickName());
            userVO.setRealName(iamInstUserInfoPO.getNickName());
            userVO.setAccount(iamInstUserInfoPO.getUsername());
        }
        Page<UserVO> page = new Page<>();
        page.setRecords(userVOS);
        page.setTotal(iamUserPageModel.getTotal());
        page.setCurrent(iamUserPageModel.getCurrent());
        page.setSize(iamUserPageModel.getSize());
        page.setPages(iamUserPageModel.getPages());
        return page;
    }

}
