/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.system.controller;


import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.core.constant.AppConstant;
import org.springblade.core.log.entity.LogApi;
import org.springblade.core.log.service.LogApiService;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;

import lombok.AllArgsConstructor;
import springfox.documentation.annotations.ApiIgnore;

/**
 * 控制器
 *
 * @author Chill
 */
@ApiIgnore
@RestController
@AllArgsConstructor
@RequestMapping(AppConstant.APPLICATION_LOG_NAME + "/api")
public class LogApiController {

	private final LogApiService logService;

	/**
	 * 查询单条
	 */
	@GetMapping("/detail")
	public R<LogApi> detail(@RequestParam Long id) {
		return R.data(logService.getById(id));
	}

	/**
	 * 查询多条(分页)
	 */
	@GetMapping("/list")
	public R<IPage<LogApi>> list(@ApiIgnore @RequestParam Map<String, Object> log, Query query) {
		int offset = (query.getCurrent() - 1) * query.getSize();
		//构造查询条件
		LambdaQueryWrapper<LogApi> queryWrapper = Condition.getQueryWrapper(log, LogApi.class).lambda().orderByDesc(LogApi::getId);
		queryWrapper.select(LogApi::getId, LogApi::getTitle, LogApi::getCreateTime, LogApi::getEnv, LogApi::getRequestUri, LogApi::getRemoteIp, LogApi::getServiceId, LogApi::getServerHost, LogApi::getServerIp);
		//分页page
		Page<LogApi> page = new Page<>(Func.toInt(query.getCurrent(), 1), Func.toInt(query.getSize(), 10));
		//当查询数据量大于5000条时，做深度分页优化
		if(offset >= 5000) {
			//查询total数据量
			long total = logService.count(Condition.getQueryWrapper(log, LogApi.class));
			//构造深度分页查询优化条件
			queryWrapper.leSql(LogApi::getId, "select id from blade_log_api order by id desc limit "+offset+", 1").last("limit " + query.getSize());
			List<LogApi> records = logService.list(queryWrapper);
			page.setTotal(total);
			page.setRecords(records);
			return R.data(page);
		}else {
			return R.data(logService.page(page, queryWrapper));
		}
	}
}
