package org.springblade.system.client;

import io.swagger.annotations.ApiOperation;
import org.springblade.common.openfeign.OpenFeignConstants;
import org.springblade.system.client.dto.IamGetUserByIdsParams;
import org.springblade.system.client.factory.IamClientFallbackFactory;
import org.springblade.system.client.factory.TenantClientFallbackFactory;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * @author hepingguo
 * @version 1.0
 * @description: TODO
 * @date 2023/11/25 11:53
 */
@FeignClient(contextId = "iamClient",
        value = OpenFeignConstants.SYSTEM_SERVICE,
        fallbackFactory = IamClientFallbackFactory.class)
public interface IamClient {


    /**
     * 通过租户id集合查询租户信息
     */
    @GetMapping( "/yx/iam/getTenantNames")
    List<TenantModel> getTenantByTenantIds(@RequestParam(value = "tenantIds", required = false) List<String> tenantIds);

    /**
     * 通过用户id获取用户信息
     * @param userId
     * @return
     */
    @GetMapping("/yx/iam/getByUserId")
    UserModel getById(@RequestParam Long userId,@RequestParam Long tenantId);

    /**
     * 通过用户id获取用户信息
     * @param params
     * @return
     */
    @PostMapping("/yx/iam/getByUserIds")
    List<UserModel> getByIds(@RequestBody IamGetUserByIdsParams params);
}
